/******************************************************************************
 *	File Name	:	DataFile.h												  *
 *	Description	:   The CDataFile class is used to create and modify		  *	
 *					data files such as *.ini and *.cfg.						  *
 *	Creator		:   Laughsky(LiQintao)										  *
 *  Create Time	:	2001/9/25												  *
 *	Bug Report	:	laugh_sky@hotmail.com || muzilee@263.net				  *
 ******************************************************************************/
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "DataFile.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDataFile::CDataFile()
{
	IsFileOpen = FALSE;
	moveLine = 1;
}

CDataFile::~CDataFile()
{

}

//Create specified file:
//The nFlags may be  1. CREATE_MODE_REBUILD
//                   2. CREATE_MODE_NOREBUILD 
BOOL CDataFile::CreateFile(LPCTSTR lpszFileName, UINT nFlags)
{
	if (!OpenFile(lpszFileName, nFlags))
		return FALSE;
	CloseFile();
	return TRUE;
}

//Read the specified line
BOOL CDataFile::ReadLine(CString &rString)
{
	if (!IsFileOpen)
		return FALSE;
	if (!file.ReadString(rString))
		return FALSE;
	return TRUE;
}

// Check if the specified file exsists
INT CDataFile::CheckFileExsist(LPCTSTR lpszFileName)
{
	if (!OpenFile(lpszFileName, OPEN_MODE_READ))
	{
		return FLAG_FILE_NOEXIST;
	}
	CloseFile();
	return FLAG_FILE_EXIST;
}

// Open the specified file
BOOL CDataFile::OpenFile(LPCTSTR lpszFileName, UINT nFlags)
{
	if (!file.Open(lpszFileName, nFlags))
		return FALSE;
	IsFileOpen = TRUE;
	FileName = lpszFileName;
	return TRUE;
}

// Close the current file
void CDataFile::CloseFile()
{
	file.Close();
	IsFileOpen = FALSE;
}

// Get the number of lines in file.
UINT CDataFile::GetLines()
{
	if (!IsFileOpen)
		return NULL;
	unsigned int  lines = 0;
	CString temp;
	MoveStart();

	// һΪգ򲻼
	while(file.ReadString(temp))
		lines ++;
	return lines;
}

// Move to the start of the file
void CDataFile::MoveStart()
{
	if (IsFileOpen)
		file.SeekToBegin();
	return;
}

// Move the end of the file
void CDataFile::MoveEnd()
{
	if (IsFileOpen)
		file.SeekToEnd();
	return;
}

// Move the pointer to the specified line, then you can 
// readoverwrite and delete the line by using ReadLine() or WriteLine(). 
BOOL CDataFile::MoveToLine(UINT line)
{
	if (!IsFileOpen)
		return FALSE;
	if (line <= 0)
	{
		moveLine = 1;
		MoveStart();
		return TRUE;
	}
	if (line > GetLines())
	{
		moveLine = GetLines()+1;
		MoveEnd();
		return TRUE;
	}
	
	CString temp;
	MoveStart();
	for (unsigned int curLine = 1; curLine < line; curLine++)
		file.ReadString(temp);
	DWORD pos = file.GetPosition();
	file.Seek(pos, CFile::begin);
	moveLine = line;
	return TRUE;
}

// Write the content you specified at current line.
// The nFlag maybe  1. WRITE_MODE_INSERT
//					2. WRITE_MODE_OVERWRITE
//					3. WRITE_MODE_DELETE
void CDataFile::WriteLine(CString &wString, UINT nFlag)
{
	UINT setLine = moveLine;
	UINT fileLen = file.GetLength();
	if (!IsFileOpen)
		return ;

	CString temp;

	MoveToLine(setLine);
	UINT upLen = file.GetPosition();
	int restLen = fileLen - upLen;
	if (restLen > FILE_MAX_LENGTH)
		return;

	CString sRestFile,sLine;
	char pbuf[FILE_MAX_LENGTH];
	memset(pbuf, 0, FILE_MAX_LENGTH);
	
	switch(nFlag)
	{
	case WRITE_MODE_INSERT:
		if (setLine <= GetLines() || GetLines() == 0)
		{
			MoveToLine(setLine);
			file.Read(pbuf, restLen);
			temp = wString+"\n"+pbuf;
			MoveToLine(setLine);
			file.WriteString(temp);
		}
		else{
			CString temp2;
			MoveToLine(GetLines());
			file.ReadString(temp2);
			temp = temp2+"\n"+wString;
			MoveToLine(GetLines());
			file.WriteString(temp);
		}
		break;
	case WRITE_MODE_OVERWRITE:
		if (setLine < GetLines())
			MoveToLine(setLine+1);
		else
		{
			setLine = GetLines();
			MoveToLine(GetLines());
			upLen = file.GetPosition();	// ָǵ򸲸һ
			MoveEnd();
		}
		file.Read(pbuf, restLen);
		if (setLine == GetLines())
			temp = wString;
		else
			temp = wString+"\n"+pbuf;

		// ΪCString㳤ʱûа"\n"ȥҪں油ĳ
		fileLen = upLen + temp.GetLength() + (GetLines()-setLine)*1;
		if (setLine < GetLines())
			MoveToLine(setLine);
		else
			MoveToLine(GetLines());
		file.WriteString(temp);
		file.SetLength(fileLen);
		break;
	case WRITE_MODE_DELETE:
		if (fileLen == 0)
			break;
		if (setLine < GetLines())
		{
			MoveToLine(setLine);
			ReadLine(wString);
			MoveToLine(setLine+1);
		}
		else
		{
			setLine = GetLines();
			MoveToLine(setLine);
			upLen = file.GetPosition();
			ReadLine(wString);
			MoveEnd();
		}
		file.Read(pbuf, restLen);
		temp = pbuf;

		// ΪCString㳤ʱûа"\n"ȥҪں油ĳ
		if (setLine < GetLines())
			fileLen = upLen + temp.GetLength() + (GetLines()-setLine -1)*1;
		else
			if (GetLines() == 1)
				fileLen = 0;
			else
				fileLen = upLen-2;// ɾһе\0x0D\0x0A
		MoveToLine(setLine);
	
		file.WriteString(temp);
		file.SetLength(fileLen);
		break;
	}
}

// Delete the specified file.
void CDataFile::DeleteFile(LPCTSTR lpszFileName)
{
	file.Remove(lpszFileName);
}
